import { liftChildren, toText } from 'myst-common';
import { remove } from 'unist-util-remove';
import { sectionTransform } from './sections.js';
/**
 * Handle sections and headers in abstract
 *
 * This removes "Abstract" title and prevents block nesting
 * inside the abstract.
 */
export function abstractTransform(tree) {
    var _a, _b;
    if (((_a = tree.children) === null || _a === void 0 ? void 0 : _a.length) === 1 && tree.children[0].type === 'sec') {
        abstractTransform(tree.children[0]);
        return;
    }
    sectionTransform(tree, 'strong');
    liftChildren(tree, 'block');
    const title = (_b = tree.children) === null || _b === void 0 ? void 0 : _b[0];
    if ((title === null || title === void 0 ? void 0 : title.type) !== 'title')
        return;
    const nextNode = tree.children[1];
    if (toText(title).toUpperCase().trim() === 'ABSTRACT') {
        title.type = '__delete__';
    }
    else if ((nextNode === null || nextNode === void 0 ? void 0 : nextNode.type) === 'p') {
        nextNode.children = [...title.children, { type: 'text', value: ' ' }, ...nextNode.children];
        title.type = '__delete__';
    }
    else {
        title.type = 'p';
    }
    remove(tree, '__delete__');
}
/**
 * Pull the first two sentences from an abstract to use as description
 *
 * The end of a sentence is defined as "<lower-case word>. <Upper-case word>"
 * This means that a name like "Mr. Smith" will not count as the end of a sentence.
 * However it also means a sentence that ends in an upper-case word (or a number)
 * will not count as a new sentence, so the description may be more than two
 * sentences.
 *
 * If the abstract is 2 sentences or less (computed as described above), the
 * entire abstract will be returned as the description.
 */
export function descriptionFromAbstract(abstract) {
    const noNewLineAbstract = abstract.replaceAll(/\s+/g, ' ');
    const sentenceRegex = /^(.*?\s[a-z]+\.)\s+([A-Z][A-Za-z]*,{0,1}\s.*)$/;
    const firstSentenceMatch = noNewLineAbstract.match(sentenceRegex);
    const firstSentence = firstSentenceMatch === null || firstSentenceMatch === void 0 ? void 0 : firstSentenceMatch[1];
    const rest = firstSentenceMatch === null || firstSentenceMatch === void 0 ? void 0 : firstSentenceMatch[2];
    if (!firstSentence || !rest)
        return noNewLineAbstract;
    const secondSentenceMatch = rest.match(sentenceRegex);
    const secondSentence = secondSentenceMatch === null || secondSentenceMatch === void 0 ? void 0 : secondSentenceMatch[1];
    if (!secondSentence)
        return noNewLineAbstract;
    return `${firstSentence} ${secondSentence}`;
}
