import { liftChildren } from 'myst-common';
import { blockNestingTransform } from 'myst-transforms';
import { select } from 'unist-util-select';
import { remove } from 'unist-util-remove';
function isSection(node) {
    return node.type === 'sec' || node.type === 'ack';
}
function recurseSections(tree, depth = 1, titleType) {
    var _a;
    const sections = (_a = tree.children) === null || _a === void 0 ? void 0 : _a.filter((n) => isSection(n));
    if (!sections || sections.length === 0)
        return;
    sections.forEach((sec) => {
        var _a, _b;
        let firstChild = (_a = sec.children) === null || _a === void 0 ? void 0 : _a[0];
        // Section labels are ignored
        if ((firstChild === null || firstChild === void 0 ? void 0 : firstChild.type) === 'label') {
            firstChild.type = '__delete__';
            firstChild = (_b = sec.children) === null || _b === void 0 ? void 0 : _b[1];
        }
        if ((firstChild === null || firstChild === void 0 ? void 0 : firstChild.type) === 'title') {
            if (titleType === 'strong') {
                firstChild.type = 'p';
                firstChild.children = [{ type: 'bold', children: firstChild.children }];
            }
            else {
                firstChild.type = 'heading';
                firstChild.id = sec.id;
                firstChild.depth = depth;
            }
        }
        recurseSections(sec, depth + 1, titleType);
    });
}
/**
 * Clean up nested sections
 *
 * - Identify sections with titles
 * - Convert those titles to headings
 * - Give headings depth value based on nesting
 * - Flatten the sections
 */
export function sectionTransform(tree, titleType) {
    var _a;
    recurseSections(tree, 1, titleType);
    remove(tree, '__delete__');
    const topSections = (_a = tree.children) === null || _a === void 0 ? void 0 : _a.filter((n) => isSection(n));
    topSections.forEach((sec) => {
        if (sec.type === 'ack')
            sec.part = 'acknowledgments';
        sec.type = 'block';
    });
    while (select('sec', tree))
        liftChildren(tree, 'sec');
    blockNestingTransform(tree);
}
export const sectionPlugin = () => (tree) => {
    sectionTransform(tree);
};
